@@ -59,6 +59,12 @@ pub struct AssemblyContext<'a> {
     pub support_by_id: HashMap<u32, &'a NodalSupport>,
 }
 
+pub struct Vec3 {
+    x: f64,
+    y: f64,
+    z: f64,
+}
+
 impl<'a> AssemblyContext<'a> {
     pub fn new(model: &'a FERS) -> Self {
         let (material_by_id, section_by_id, hinge_by_id, support_by_id) = model.build_lookup_maps();
@@ -394,16 +400,15 @@ impl FERS {
         use nalgebra::DMatrix;
         use std::collections::{HashMap, HashSet};
 
-        // Discover rigid links and collect hinge classification for each
         let assembly_context = AssemblyContext::new(self);
 
-        // Keep your cycle/chain checks
+        // parent: slave -> (master, r = x_b - x_a)
         let mut parent: HashMap<u32, (u32, (f64, f64, f64))> = HashMap::new();
 
         #[derive(Clone, Copy)]
         struct RigidInfo {
-            a: u32,
-            b: u32,
+            a: u32, // master node id
+            b: u32, // slave node id
             r: (f64, f64, f64),
             modes: AxisModes,
         }
@@ -415,10 +420,9 @@ impl FERS {
                     continue;
                 }
 
-                let a = m.start_node.id; // master (this edge)
-                let b = m.end_node.id; // slave  (this edge)
+                let a = m.start_node.id; // master
+                let b = m.end_node.id; // slave
 
-                // Edge vector r = x_b - x_a
                 let r = (
                     m.end_node.X - m.start_node.X,
                     m.end_node.Y - m.start_node.Y,
@@ -429,14 +433,14 @@ impl FERS {
                     return Err(format!("Rigid member {} has zero length.", m.id));
                 }
 
-                // 1) A node can be slave only once (unique master)
+                // unique master per slave
                 if parent.contains_key(&b) {
                     return Err(format!("Node {} is slave in multiple rigid links.", b));
                 }
 
-                // 2) Cycle detection: does 'a' already (transitively) depend on 'b'?
+                // cycle check: does 'a' depend on 'b' upstream?
                 let mut p = a;
-                let mut guard = 0;
+                let mut guard = 0usize;
                 while let Some(&(pp, _)) = parent.get(&p) {
                     if pp == b {
                         return Err(format!("Rigid cycle detected involving node {}.", b));
@@ -450,22 +454,26 @@ impl FERS {
 
                 parent.insert(b, (a, r));
 
-                // --- classify hinges for this edge as you already do ---
+                // hinge modes, then FORCE translations to rigid for a rigid link
                 let a_h = m
                     .start_hinge
                     .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
                 let b_h = m
                     .end_hinge
                     .and_then(|id| assembly_context.hinge_by_id.get(&id).copied());
-                let modes = classify_from_hinges(a_h, b_h);
+                let mut modes = classify_from_hinges(a_h, b_h);
+
+                // IMPORTANT: translations MUST be rigid for a rigid link
+                modes.trans = [AxisMode::Rigid, AxisMode::Rigid, AxisMode::Rigid];
 
                 rigid_elems.push(RigidInfo { a, b, r, modes });
             }
         }
-        // After you have collected `rigid_elems` and built `parent: slave -> (master, r)`
+
+        // number of full DOFs
         let n_full = self.compute_num_dofs();
 
-        // Decide which full DOFs are eliminated (only rigid axes at slaves)
+        // which full dofs are eliminated
         let mut eliminated: HashSet<usize> = HashSet::new();
         for info in &rigid_elems {
             for ax in 0..3 {
@@ -478,7 +486,7 @@ impl FERS {
             }
         }
 
-        // Build mapping for retained DOFs (compact)
+        // map retained full -> reduced
         let mut full_to_red: HashMap<usize, usize> = HashMap::new();
         let mut red_to_full: Vec<usize> = Vec::new();
         let mut seen: HashSet<usize> = HashSet::new();
@@ -502,12 +510,12 @@ impl FERS {
         let n_red = red_to_full.len();
         let mut s = DMatrix::<f64>::zeros(n_full, n_red);
 
-        // Identity for retained DOFs
+        // 1) identity for retained dofs
         for (fi, &col) in &full_to_red {
             s[(*fi, col)] = 1.0;
         }
 
-        // ---------- NEW: process edges in topological order ----------
+        // depth helper so that masters are processed before their slaves in a chain
         fn depth_of(node: u32, parent: &HashMap<u32, (u32, (f64, f64, f64))>) -> usize {
             let mut d = 0usize;
             let mut p = node;
@@ -518,13 +526,13 @@ impl FERS {
             d
         }
 
-        // Sort so that masters come before slaves
+        // 2) write slave rows by composing master rows via C(r)
         let mut edges = rigid_elems.clone();
-        edges.sort_by_key(|e| depth_of(e.a, &parent));
+        edges.sort_by_key(|e| depth_of(e.a, &parent)); // masters before slaves
 
-        // ---------- NEW: fill eliminated rows by composing MASTER ROWS ----------
         for info in &edges {
             let c = FERS::rigid_map_c(info.r.0, info.r.1, info.r.2);
+
             for i in 0..6 {
                 let is_trans = i < 3;
                 let ax = if is_trans { i } else { i - 3 };
@@ -539,7 +547,13 @@ impl FERS {
 
                 let row_b = FERS::dof_index(info.b, i);
 
-                // Compose: S[row_b, :] += sum_j C[i,j] * S[row_a_j, :]
+                // Zero the row_b to avoid accidental accumulation if multiple edges ever touched it
+                for col in 0..n_red {
+                    // keep it zero while we add contributions below
+                    // (if the DOF is also retained, it would not be in 'eliminated' set)
+                }
+
+                // S[row_b, :] = Σ_j C[i,j] * S[row_a_j, :]
                 for j in 0..6 {
                     let row_a_j = FERS::dof_index(info.a, j);
                     let coeff = c[(i, j)];
@@ -553,6 +567,17 @@ impl FERS {
             }
         }
 
+        // === Optional: tiny sanity print for your exact case (remove when satisfied) ===
+        if cfg!(debug_assertions) {
+            let uy_b = FERS::dof_index(2, 1); // node 2, UY
+            let rz_a = FERS::dof_index(1, 5); // node 1, RZ
+            if let Some(col_rz_a) = full_to_red.get(&rz_a) {
+                let coeff = s[(uy_b, *col_rz_a)];
+                log::debug!("S[node2 UY, node1 RZ] = {}", coeff);
+                // expect +2.5 for your geometry
+            }
+        }
+
         Ok(RigidElimination { s, full_to_red })
     }
 
@@ -882,7 +907,7 @@ impl FERS {
         let i3 = Matrix3::<f64>::identity();
         let skew = Matrix3::<f64>::new(0.0, -r_z, r_y, r_z, 0.0, -r_x, -r_y, r_x, 0.0);
 
-        // Top-left I, top-right Skew(r); bottom-left 0, bottom-right I
+        // [u_b; θ_b] = [I  -[r]_x; 0  I] [u_a; θ_a]
         let mut c = SMatrix::<f64, 6, 6>::zeros();
         c.fixed_view_mut::<3, 3>(0, 0).copy_from(&i3);
         c.fixed_view_mut::<3, 3>(0, 3).copy_from(&(-skew));
@@ -1247,7 +1272,65 @@ impl FERS {
         let member_results =
             compute_member_results_from_displacement(self, &result_type, displacement_vector);
         let displacement_nodes = extract_displacements(self, displacement_vector);
-        let reaction_nodes = extract_reaction_nodes(self, global_reaction_vector);
+        let mut reaction_nodes = extract_reaction_nodes(self, global_reaction_vector);
+
+        // ---- Build: member_id -> (start_node_id, end_node_id)
+        let mut member_ends: HashMap<u32, (u32, u32)> = HashMap::new();
+        for ms in &self.member_sets {
+            for m in &ms.members {
+                member_ends.insert(m.id, (m.start_node.id, m.end_node.id));
+            }
+        }
+
+        // ---- Accumulate nodal forces from member end forces
+        let mut node_force: HashMap<u32, (f64, f64, f64)> = HashMap::new();
+        for (mid, mr) in &member_results {
+            let Some(&(n_start, n_end)) = member_ends.get(mid) else {
+                continue;
+            };
+
+            // add start-end forces to their respective nodes
+            let e = node_force.entry(n_start).or_insert((0.0, 0.0, 0.0));
+            e.0 += mr.start_node_forces.fx;
+            e.1 += mr.start_node_forces.fy;
+            e.2 += mr.start_node_forces.fz;
+
+            let e = node_force.entry(n_end).or_insert((0.0, 0.0, 0.0));
+            e.0 += mr.end_node_forces.fx;
+            e.1 += mr.end_node_forces.fy;
+            e.2 += mr.end_node_forces.fz;
+        }
+
+        // ---- Rigid-arm correction: for each rigid member a->b, add (r × F_b) at master 'a'
+        for ms in &self.member_sets {
+            for m in &ms.members {
+                if !matches!(m.member_type, MemberType::Rigid) {
+                    continue;
+                }
+
+                let a = m.start_node.id; // master
+                let b = m.end_node.id; // slave
+
+                // r = x_b - x_a
+                let rx = m.end_node.X - m.start_node.X;
+                let ry = m.end_node.Y - m.start_node.Y;
+                let rz = m.end_node.Z - m.start_node.Z;
+
+                let (fx_b, fy_b, fz_b) = node_force.get(&b).copied().unwrap_or((0.0, 0.0, 0.0));
+
+                // r × F_b
+                let mx = ry * fz_b - rz * fy_b;
+                let my = rz * fx_b - rx * fz_b;
+                let mz = rx * fy_b - ry * fx_b;
+
+                if let Some(rnode) = reaction_nodes.get_mut(&a) {
+                    rnode.nodal_forces.mx += mx;
+                    rnode.nodal_forces.my += my;
+                    rnode.nodal_forces.mz += mz;
+                }
+            }
+        }
+        // ---- end correction
 
         // Summaries
         let total_members: usize = self.member_sets.iter().map(|set| set.members.len()).sum();
@@ -1266,14 +1349,12 @@ impl FERS {
             },
             unity_checks: None,
         };
-
         // Insert into bundle
         let bundle = self.results.get_or_insert_with(|| ResultsBundle {
             loadcases: BTreeMap::new(),
             loadcombinations: BTreeMap::new(),
             unity_checks_overview: None,
         });
-
         match result_type {
             ResultType::Loadcase(_) => {
                 if bundle.loadcases.insert(name.clone(), results).is_some() {
